from si_tests import logger

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from si_tests.managers.kaas_manager import Cluster

LOG = logger.logger


class ClusterCheckMosManager(object):
    def __init__(self, cluster: "Cluster"):
        self._cluster: "Cluster" = cluster

    @property
    def cluster(self) -> "Cluster":
        return self._cluster

    def check_vrouter_pods(self, tf_pod_pref, os_manager):
        tf_ns = os_manager.tf_namespace
        ds_list = os_manager.api.daemonsets.list_starts_with(tf_pod_pref, tf_ns)
        LOG.info(f"Total vRouter DaemonSets: {len(ds_list)}")
        # container name for validation
        c_name = 'agent'
        for ds in ds_list:
            ds_data = ds.read()
            status = ds_data.status
            LOG.info(f"Verify DaemonSet {ds.name}: {status.number_ready}"
                     f"/{status.desired_number_scheduled} (ready/desired)")
            # Skip DaemonSets without scheduled pods
            if status.desired_number_scheduled == 0:
                continue
            pod_list = os_manager.api.pods.list_starts_with(ds.name, tf_ns)
            pods = [pod for pod in pod_list if pod.get_owner_reference() ==
                    f"DaemonSet/{ds.name}"]
            containers = ds_data.spec.template.spec.containers
            ds_c_spec_list = [c for c in containers if c.name == c_name]
            if not ds_c_spec_list:
                raise Exception(
                    f"DaemonSet {ds.name}: container {c_name} "
                    f"wasn't found in template")
            ds_c_spec = ds_c_spec_list.pop()
            pod_containers = []
            pod_name = None
            for pod in pods:
                pod_data = pod.read()
                pod_containers = [c for c in pod_data.spec.containers if
                                  c.name ==
                                  c_name]
                if pod_containers:
                    pod_name = pod.name
                    break
            if not pod_containers:
                raise Exception(
                    f"Container {c_name} wasn't found in pods belongs "
                    f"to DaemonSet {ds.name}")
            container = pod_containers.pop()
            LOG.info(f"Check {c_name} image in pod {pod_name} belonged to "
                     f"DaemonSet {ds.name}:\n"
                     f"DaemonSet image: {ds_c_spec.image}\n"
                     f"Pod image: {container.image}")
            assert container.image == ds_c_spec.image, \
                f"vRouter pod {pod_name} wasn't updated.\n" \
                f"{container.image} != {ds_c_spec.image}"
